@@ -491,7 +491,9 @@ common_tab_updateGasData(int update_tab, char *car)
    if (update_tab)
    {
       entry_units = returnEntryUnits(car);
-
+      /* IF entry_units not defined then use display units */
+      if (entry_units == -1)
+         entry_units = returnUnits();
       date_string = "Date";
       odo_string = "Odometer";
       cost_string = "Cost";
