@@ -1,5 +1,5 @@
 /*
- * Copyright: (C) 2008 Bruce W. Forsberg
+ * Copyright: (C) 2010 Bruce W. Forsberg
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -64,20 +64,20 @@ static void graph_it(int year, GtkWidget *w);
 static void graph_mileage_year(int year, GtkWidget *w);
 static void graph_distance_year(int year, GtkWidget *w);
 static void graph_cost_year(int year, GtkWidget *w);
-static int  graph_get_max_distance();
-static int  graph_get_max_mileage();
-static int  graph_get_max_cost();
+static void graph_get_max_values(int*, int*, int*);
 
-#define X_OFFSET 70
+
+#define X_OFFSET 80
 #define X_DELTA  40
 #define X_NUM    13
-#define X_MAX_NUM X_NUM * X_DELTA + X_OFFSET
 
-#define Y_OFFSET 20
+#define Y_OFFSET 30
 #define Y_DELTA  23
 #define Y_NUM    11
 #define Y_MAX_NUM Y_NUM * Y_DELTA + Y_OFFSET
 
+#define YEAR_SPACING 65
+
 #define MILEAGE_STR  "Mileage"
 #define DISTANCE_STR "Distance"
 #define COST_STR     "Cost"
@@ -86,6 +86,13 @@ static int  graph_get_max_cost();
 #define GRAPH_HEIGHT   300
 #define GRAPH_WIDTH    600
 
+static int width_of_graph = GRAPH_WIDTH;
+static int height_of_graph = GRAPH_HEIGHT;
+static int x_delta = X_DELTA;
+static int y_delta = Y_DELTA;
+static int x_max_num = X_NUM * X_DELTA + X_OFFSET;
+static int y_max_num = Y_NUM * Y_DELTA + Y_OFFSET;
+
 
 void graph_tab_layout(GtkWidget *window)
 {
@@ -123,7 +130,7 @@ void graph_tab_layout(GtkWidget *window)
    w_area = gtk_drawing_area_new();
    gtk_table_attach(GTK_TABLE(table1), w_area, 0, 9, 2, 9, GTK_FILL, GTK_FILL, 2, 2);
    /* TBD - need to fill rest of space and get dimensions */
-   gtk_widget_set_size_request(w_area, GRAPH_WIDTH, GRAPH_HEIGHT);
+   gtk_widget_set_size_request(w_area, width_of_graph, height_of_graph);
    g_signal_connect(G_OBJECT(w_area), "expose_event", G_CALLBACK(graph_expose_event), NULL);
 }
 
@@ -240,7 +247,7 @@ car_selector_clicked_cb(GtkWidget* w, gpointer data)
          free(car);
    }
    if (w_area != NULL)
-      gtk_widget_queue_draw_area(w_area, 0, 0, GRAPH_WIDTH, GRAPH_HEIGHT);
+      gtk_widget_queue_draw_area(w_area, 0, 0, width_of_graph, height_of_graph);
 }
 
 void
@@ -251,7 +258,7 @@ graph_selector1_clicked_cb(GtkWidget* w, gpointer data)
 #endif
 
    if (w_area != NULL)
-      gtk_widget_queue_draw_area(w_area, 0, 0, GRAPH_WIDTH, GRAPH_HEIGHT);
+      gtk_widget_queue_draw_area(w_area, 0, 0, width_of_graph, height_of_graph);
 }
 
 void
@@ -262,7 +269,7 @@ graph_selector2_clicked_cb(GtkWidget* w, gpointer data)
 #endif
 
    if (w_area != NULL)
-      gtk_widget_queue_draw_area(w_area, 0, 0, GRAPH_WIDTH, GRAPH_HEIGHT);
+      gtk_widget_queue_draw_area(w_area, 0, 0, width_of_graph, height_of_graph);
 }
 
 gboolean
@@ -297,9 +304,9 @@ graph_expose_event(
       color[4].red = 0;
       color[4].blue = 65535;
       color[4].green = 65535;
-      color[5].red = 65535;
+      color[5].red = 55535;
       color[5].blue = 0;
-      color[5].green = 65535;
+      color[5].green = 55535;
       color[6].red = 32000;
       color[6].blue = 32000;
       color[6].green = 32000;
@@ -403,7 +410,7 @@ graph_draw_legend(
       pango_layout_set_text(layout, number, -1);
       gdk_draw_layout(area->window, area->style->fg_gc[area->state],
                    offset, 0, layout);
-      offset += 55;
+      offset += YEAR_SPACING;
    }
 }
 
@@ -418,18 +425,18 @@ graph_draw_grid(GtkWidget *area)
    for (y = 0; y < Y_NUM; y++)
    {
       gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
-         X_OFFSET, Y_OFFSET + Y_DELTA * y,
-         X_OFFSET + (X_NUM - 1) * X_DELTA, Y_OFFSET + Y_DELTA * y);
+         X_OFFSET, Y_OFFSET + y_delta * y,
+         X_OFFSET + (X_NUM - 1) * x_delta, Y_OFFSET + y_delta * y);
 #if 0
-      fprintf(stdout, "Draw Grid line %d %d %d %d\n", X_OFFSET, Y_OFFSET + Y_DELTA * y, X_OFFSET + (X_NUM - 1) * X_DELTA, Y_OFFSET + Y_DELTA * y);
+      fprintf(stdout, "Draw Grid line %d %d %d %d\n", X_OFFSET, Y_OFFSET + y_delta * y, X_OFFSET + (X_NUM - 1) * x_delta, Y_OFFSET + y_delta * y);
 #endif
    }
 
    for (x = 0; x < X_NUM; x++)
    {
       gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
-         X_OFFSET + X_DELTA * x, Y_OFFSET,
-         X_OFFSET + X_DELTA * x, Y_OFFSET + (Y_NUM - 1) * Y_DELTA);
+         X_OFFSET + x_delta * x, Y_OFFSET,
+         X_OFFSET + x_delta * x, Y_OFFSET + (Y_NUM - 1) * y_delta);
    }
 }
 
@@ -455,76 +462,76 @@ graph_draw_text(GtkWidget *area)
 
    pango_layout_set_text(layout, "Jan", -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   X_OFFSET + 0 * X_DELTA - (X_DELTA / 2),
-                   (Y_NUM * Y_DELTA), layout);
+                   X_OFFSET + 0 * x_delta - (x_delta / 2),
+                   (Y_NUM - 1) * y_delta + Y_OFFSET, layout);
 
    pango_layout_set_text(layout, "Apr", -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   X_OFFSET + 3 * X_DELTA - (X_DELTA / 2),
-                   (Y_NUM * Y_DELTA), layout);
+                   X_OFFSET + 3 * x_delta - (x_delta / 2),
+                   (Y_NUM - 1) * y_delta + Y_OFFSET, layout);
 
    pango_layout_set_text(layout, "Jul", -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   X_OFFSET + 6 * X_DELTA - (X_DELTA / 2),
-                   (Y_NUM * Y_DELTA), layout);
+                   X_OFFSET + 6 * x_delta - (x_delta / 2),
+                   (Y_NUM - 1) * y_delta + Y_OFFSET, layout);
 
    pango_layout_set_text(layout, "Oct", -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   X_OFFSET + 9 * X_DELTA - (X_DELTA / 2),
-                   (Y_NUM * Y_DELTA), layout);
+                   X_OFFSET + 9 * x_delta - (x_delta / 2),
+                   (Y_NUM - 1) * y_delta + Y_OFFSET, layout);
 
    pango_layout_set_text(layout, "Jan", -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   X_OFFSET + 12 * X_DELTA - (X_DELTA / 2),
-                   (Y_NUM * Y_DELTA), layout);
+                   X_OFFSET + 12 * x_delta - (x_delta / 2),
+                   (Y_NUM - 1) * y_delta + Y_OFFSET, layout);
 
    pango_layout_set_text(layout, "0", -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   1, ((Y_NUM-1) * Y_DELTA) + Y_DELTA/2, layout);
+                   1, ((Y_NUM-1) * y_delta) + y_delta/2 + Y_OFFSET - 20, layout);
 
    sprintf(one, "%-7.2f", max_display / 10.0 * 2.0);
    pango_layout_set_text(layout, one, -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   1, ((Y_NUM-3) * Y_DELTA) + Y_DELTA/2, layout);
+                   1, ((Y_NUM-3) * y_delta) + y_delta/2 + Y_OFFSET - 20, layout);
 
    sprintf(two, "%-7.2f", max_display / 10.0 * 4.0);
    pango_layout_set_text(layout, two, -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   1, ((Y_NUM-5) * Y_DELTA) + Y_DELTA/2, layout);
+                   1, ((Y_NUM-5) * y_delta) + y_delta/2 + Y_OFFSET - 20, layout);
 
    sprintf(three, "%-7.2f", max_display / 10.0 * 6.0);
    pango_layout_set_text(layout, three, -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   1, ((Y_NUM-7) * Y_DELTA) + Y_DELTA/2, layout);
+                   1, ((Y_NUM-7) * y_delta) + y_delta/2 + Y_OFFSET - 20, layout);
 
    sprintf(four, "%-7.2f", max_display / 10.0 * 8.0);
    pango_layout_set_text(layout, four, -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   1, ((Y_NUM-9) * Y_DELTA) + Y_DELTA/2, layout);
+                   1, ((Y_NUM-9) * y_delta) + y_delta/2 + Y_OFFSET - 20, layout);
 
    sprintf(five, "%-7.2f", (double)max_display);
    pango_layout_set_text(layout, five, -1);
    gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                   1, ((Y_NUM-11) * Y_DELTA) + Y_DELTA/2, layout);
+                   1, ((Y_NUM-11) * y_delta) + y_delta/2 + Y_OFFSET - 20, layout);
 
    graphtype = gtk_combo_box_get_active_text(GTK_COMBO_BOX(graph_selector1));
    if (strncmp(graphtype, MILEAGE_STR, strlen(MILEAGE_STR)) == 0)
    {
       pango_layout_set_text(layout, mile_str, -1);
       gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                      0, (Y_NUM * Y_DELTA) / 2, layout);
+                      0, (Y_NUM * y_delta) / 2, layout);
    }
    else if (strncmp(graphtype, DISTANCE_STR, strlen(DISTANCE_STR)) == 0)
    {
       pango_layout_set_text(layout, dist_str, -1);
       gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                      0, (Y_NUM * Y_DELTA) / 2, layout);
+                      0, (Y_NUM * y_delta) / 2, layout);
    }
    else if (strncmp(graphtype, COST_STR, strlen(COST_STR)) == 0)
    {
       pango_layout_set_text(layout, cost_unit_str, -1);
       gdk_draw_layout(area->window, area->style->fg_gc[area->state],
-                      0, (Y_NUM * Y_DELTA) / 2, layout);
+                      0, (Y_NUM * y_delta) / 2, layout);
    }
 
    if (graphtype)
@@ -639,12 +646,10 @@ graph_tab_load_data(char *car)
    }
 
    /* get max values for graph scale */
-   max_distance = graph_get_max_distance();
-   max_mileage  = graph_get_max_mileage();
-   max_cost     = graph_get_max_cost();
+   graph_get_max_values(&max_distance, &max_mileage, &max_cost);
 
    if (w_area != NULL)
-      gtk_widget_queue_draw_area(w_area, 0, 0, GRAPH_WIDTH, GRAPH_HEIGHT);
+      gtk_widget_queue_draw_area(w_area, 0, 0, width_of_graph, height_of_graph);
 }
 
 void
@@ -691,11 +696,15 @@ graph_mileage_year(
    int      x_val, y_val;
    int      p_x = -1;
    int      p_y = -1;
-   int      y_max = Y_OFFSET + (Y_NUM - 1) * Y_DELTA;
+   int      y_max = Y_OFFSET + (Y_NUM - 1) * y_delta;
    int      days_in_month;
    char    *saveptr;
    char    *cost_str;
    char    *car;
+   int      process_after = 0;
+   int      process_before = 0;
+   double   y_m;
+   double   x_m;
 
    car = gtk_combo_box_get_active_text(GTK_COMBO_BOX(car_selector));
    if (car[strlen(car) - 1] == '\n')
@@ -733,41 +742,61 @@ graph_mileage_year(
 
       days_in_month = g_date_get_days_in_month(g_date_get_month(&gdate), g_date_get_year(&gdate));
 
-      if (g_date_get_year(&gdate) == year)
+      /* TBD - need to handle P entries to not draw lines */
+      /* IF no previous entry then just save off previous values */
+      if (strcmp(p, "1") == 0)
       {
-         /* IF no previous entry then just save off previous values */
-         if (strcmp(p, "1") == 0)
-         {
-         }
-         else if (prev_odo == -1)
-         {
-         }
+         p_x = -1;
+         p_y = -1;
+         process_before = 0;
+
+         /* If the first entry after Dec 31 for requested year then don't process a line */
+         if (g_date_get_year(&gdate) > year)
+            process_after = 1;
+      }
+      else if (prev_odo == -1)
+      {
+      }
+      else
+      {
+         if (entry_units == CAR_MPG_US_INDEX)
+            mpg = (odometer - prev_odo) / fuel;
+         if (entry_units == CAR_MPG_IMP_INDEX)
+            mpg = (odometer - prev_odo) / fuel;
+         else if (entry_units == CAR_LP100KM_INDEX)
+            mpg = 100.0 * fuel / (odometer - prev_odo);
+         else if (entry_units == CAR_LP10KM_INDEX)
+            mpg = 10.0 * fuel / (odometer - prev_odo);
+         else if (entry_units == CAR_MPL_INDEX)
+            mpg = (odometer - prev_odo) / fuel;
+         else if (entry_units == CAR_KMPL_INDEX)
+            mpg = (odometer - prev_odo) / fuel;
          else
+            mpg = (odometer - prev_odo) / fuel;
+         convertMPG(mpg, &mpg, entry_units, units);
+
+         y_val = y_max - (int)(mpg * ((y_max - Y_OFFSET) / (double)max_display));
+         x_val = ((int)g_date_get_month(&gdate) - 1) * x_delta + X_OFFSET;
+         x_val = x_val + (x_delta * g_date_get_day(&gdate) / days_in_month);
+
+         if (process_before == 1)
          {
-            if (entry_units == CAR_MPG_US_INDEX)
-               mpg = (odometer - prev_odo) / fuel;
-            if (entry_units == CAR_MPG_IMP_INDEX)
-               mpg = (odometer - prev_odo) / fuel;
-            else if (entry_units == CAR_LP100KM_INDEX)
-               mpg = 100.0 * fuel / (odometer - prev_odo);
-            else if (entry_units == CAR_LP10KM_INDEX)
-               mpg = 10.0 * fuel / (odometer - prev_odo);
-            else if (entry_units == CAR_MPL_INDEX)
-               mpg = (odometer - prev_odo) / fuel;
-            else if (entry_units == CAR_KMPL_INDEX)
-               mpg = (odometer - prev_odo) / fuel;
-            else
-               mpg = (odometer - prev_odo) / fuel;
-            convertMPG(mpg, &mpg, entry_units, units);
+            y_m = y_val - p_y;
+            x_m = x_val - p_x;
 
-            y_val = y_max - (int)(mpg * ((y_max - Y_OFFSET) / (double)max_display));
-            x_val = ((int)g_date_get_month(&gdate) - 1) * X_DELTA + X_OFFSET;
-            x_val = x_val + (X_DELTA * g_date_get_day(&gdate) / days_in_month);
+            /* y = mx + b */
+            p_y = (y_m / x_m) * (X_OFFSET - p_x) + p_y;
+            p_x = X_OFFSET; 
 
+            process_before = 0;
+         }
+            
+         if (g_date_get_year(&gdate) == year)
+         {
             if (p_x != -1)
             {
-               x_val = CLAMP(x_val, 0, X_MAX_NUM);
-               y_val = CLAMP(y_val, 0, Y_MAX_NUM);
+               x_val = CLAMP(x_val, 0, x_max_num);
+               y_val = CLAMP(y_val, 0, y_max_num);
 
                gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
                   p_x, p_y, x_val, y_val);
@@ -775,10 +804,38 @@ graph_mileage_year(
       fprintf(stdout, "Draw Mileage line %d %d %d %d\n", p_x, p_y, x_val, y_val);
 #endif
             }
-            p_x = x_val;
-            p_y = y_val;
          }
+         /* We need to process the Jan 1 item for mileage */
+         else if (g_date_get_year(&gdate) < year)
+         {
+            /* Get an X location that is before Jan 1 */
+            x_val = ((int)g_date_get_month(&gdate) - 12 - 1) * x_delta + X_OFFSET;
+            x_val = x_val + (x_delta * g_date_get_day(&gdate) / days_in_month);
+            process_before = 1;
+         }
+         /* We need to process the Dec 31 item for mileage */
+         else if (g_date_get_year(&gdate) > year)
+         {
+            x_val = X_OFFSET + 12 * x_delta;
+            /* Process first entry of next year if we processed entries for last year */
+            if ((process_after == 0) && (p_x != -1))
+            {
+               /* Get an X location that extents beyond Dec 31 */
+               x_val = ((int)g_date_get_month(&gdate) + 12 - 1) * x_delta + X_OFFSET;
+               x_val = x_val + (x_delta * g_date_get_day(&gdate) / days_in_month);
+               y_m = y_val - p_y;
+               x_m = x_val - p_x;
 
+               /* y = mx + b */
+               y_val = (y_m / x_m) * (X_OFFSET + 12 * x_delta - p_x) + p_y;
+               x_val = X_OFFSET + 12 * x_delta; 
+               gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
+                  p_x, p_y, x_val, y_val);
+               process_after = 1;
+            }
+         }
+         p_x = x_val;
+         p_y = y_val;
       }
       prev_odo = odometer;
       prev_fuel = fuel;
@@ -811,10 +868,14 @@ graph_distance_year(
    int      p_y = -1;
    int      miles;
    int      days;
-   int      y_max = Y_OFFSET + (Y_NUM - 1) * Y_DELTA;
+   int      y_max = Y_OFFSET + (Y_NUM - 1) * y_delta;
    int      days_in_month;
    char    *saveptr;
    char    *cost_str;
+   int      process_after = 0;
+   int      process_before = 0;
+   double   y_m;
+   double   x_m;
 
 
    data_item = g_string_new(NULL);
@@ -841,30 +902,43 @@ graph_distance_year(
 
       days_in_month = g_date_get_days_in_month(g_date_get_month(&gdate), g_date_get_year(&gdate));
 
-      if (g_date_get_year(&gdate) == year)
+      /* TBD - need to handle P entries to not draw lines */
+      /* IF no previous entry then just save off previous values */
+      if (strcmp(p, "1") == 0)
+      { 
+      }
+      else if (prev_odo == -1)
       {
-         /* IF no previous entry then just save off previous values */
-         if (strcmp(p, "1") == 0)
-         { 
-         }
-         else if (prev_odo == -1)
-         {
-         }
-         else
-         {
-            miles = odometer - prev_odo;
-            days = g_date_days_between(&prev_date, &gdate);
-            /* IF fillup on same day then days to 1 */
-            if (days == 0)
-               days = 1;
-            y_val = y_max - (int)((double)miles / (double)days * (y_max - Y_OFFSET) / (double)max_display);
-            x_val = ((int)g_date_get_month(&gdate) - 1) * X_DELTA + X_OFFSET;
-            x_val = x_val + (X_DELTA * g_date_get_day(&gdate) / days_in_month);
+      }
+      else
+      {
+         miles = odometer - prev_odo;
+         days = g_date_days_between(&prev_date, &gdate);
+         /* IF fillup on same day then days to 1 */
+         if (days == 0)
+            days = 1;
+         y_val = y_max - (int)((double)miles / (double)days * (y_max - Y_OFFSET) / (double)max_display);
+         x_val = ((int)g_date_get_month(&gdate) - 1) * x_delta + X_OFFSET;
+         x_val = x_val + (x_delta * g_date_get_day(&gdate) / days_in_month);
 
+         if (process_before == 1)
+         {
+            y_m = y_val - p_y;
+            x_m = x_val - p_x;
+
+            /* y = mx + b */
+            p_y = (y_m / x_m) * (X_OFFSET - p_x) + p_y;
+            p_x = X_OFFSET;
+
+            process_before = 0;
+         }
+
+         if (g_date_get_year(&gdate) == year)
+         {
             if (p_x != -1)
             {
-               x_val = CLAMP(x_val, 0, X_MAX_NUM);
-               y_val = CLAMP(y_val, 0, Y_MAX_NUM);
+               x_val = CLAMP(x_val, 0, x_max_num);
+               y_val = CLAMP(y_val, 0, y_max_num);
 
                gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
                   p_x, p_y, x_val, y_val);
@@ -872,10 +946,38 @@ graph_distance_year(
       fprintf(stdout, "Draw distance line %d %d %d %d\n", p_x, p_y, x_val, y_val);
 #endif
             }
-            p_x = x_val;
-            p_y = y_val;
          } 
+         /* We need to process the Jan 1 item for mileage */
+         else if (g_date_get_year(&gdate) < year)
+         {
+            /* Get an X location that is before Jan 1 */
+            x_val = ((int)g_date_get_month(&gdate) - 12 - 1) * x_delta + X_OFFSET;
+            x_val = x_val + (x_delta * g_date_get_day(&gdate) / days_in_month);
+            process_before = 1;
+         }
+         /* We need to process the Dec 31 item for mileage */
+         else if (g_date_get_year(&gdate) > year)
+         {
+            x_val = X_OFFSET + 12 * x_delta;
+            /* Process first entry of next year if we processed entries for last year */
+            if ((process_after == 0) && (p_x != -1))
+            {
+               /* Get an X location that extents beyond Dec 31 */
+               x_val = ((int)g_date_get_month(&gdate) + 12 - 1) * x_delta + X_OFFSET;
+               x_val = x_val + (x_delta * g_date_get_day(&gdate) / days_in_month);
+               y_m = y_val - p_y;
+               x_m = x_val - p_x;
 
+               /* y = mx + b */
+               y_val = (y_m / x_m) * (X_OFFSET + 12 * x_delta - p_x) + p_y;
+               x_val = X_OFFSET + 12 * x_delta;
+               gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
+                  p_x, p_y, x_val, y_val);
+               process_after = 1;
+            }
+         }
+         p_x = x_val;
+         p_y = y_val;
       } 
       prev_odo = odometer;
       prev_date = gdate;
@@ -901,13 +1003,17 @@ graph_cost_year(
    int      x_val, y_val;
    int      p_x = -1;
    int      p_y = -1;
-   int      y_max = Y_OFFSET + (Y_NUM - 1) * Y_DELTA;
+   int      y_max = Y_OFFSET + (Y_NUM - 1) * y_delta;
    int      days_in_month;
    char    *saveptr;
    char    *cost_str;
    int      cost_units;
    int      first_pass = TRUE;
    double   cost_total;
+   int      process_after = 0;
+   int      process_before = 0;
+   double   y_m;
+   double   x_m;
 
 
    data_item = g_string_new(NULL);
@@ -939,39 +1045,51 @@ graph_cost_year(
 
       days_in_month = g_date_get_days_in_month(g_date_get_month(&gdate), g_date_get_year(&gdate));
    
-      if (g_date_get_year(&gdate) == year)
+      /* IF no previous entry then just save off previous values */
+      if (strcmp(p, "1") == 0)
       {
-         /* IF no previous entry then just save off previous values */
-         if (strcmp(p, "1") == 0)
+      }
+      else if (first_pass == TRUE)
+      {
+         first_pass = FALSE;
+      }
+      else
+      {
+         if (cost_units == 0)
          {
-         }
-         else if (first_pass == TRUE)
-         {
-            first_pass = FALSE;
+            if (fuel == 0)
+               cost_total = 0;
+            else
+               cost_total = cost / fuel;
          }
          else
+            cost_total = cost;
+
+         /* Only if cost entry exists */
+         if (cost_total > 0.000000001)
          {
-            if (cost_units == 0)
+            y_val = y_max - (int)(cost_total * (y_max - Y_OFFSET) / (double)max_display);
+            x_val = ((int)g_date_get_month(&gdate) - 1) * x_delta + X_OFFSET;
+            x_val = x_val + (x_delta * g_date_get_day(&gdate) / days_in_month);
+
+            if (process_before == 1)
             {
-               if (fuel == 0)
-                  cost_total = 0;
-               else
-                  cost_total = cost / fuel;
+               y_m = y_val - p_y;
+               x_m = x_val - p_x;
+
+               /* y = mx + b */
+               p_y = (y_m / x_m) * (X_OFFSET - p_x) + p_y;
+               p_x = X_OFFSET;
+
+               process_before = 0;
             }
-            else
-               cost_total = cost;
 
-            /* Only if cost entry exists */
-            if (cost_total > 0.000000001)
+            if (g_date_get_year(&gdate) == year)
             {
-               y_val = y_max - (int)(cost_total * (y_max - Y_OFFSET) / (double)max_display);
-               x_val = ((int)g_date_get_month(&gdate) - 1) * X_DELTA + X_OFFSET;
-               x_val = x_val + (X_DELTA * g_date_get_day(&gdate) / days_in_month);
-
                if (p_x != -1)
                {
-                  x_val = CLAMP(x_val, 0, X_MAX_NUM);
-                  y_val = CLAMP(y_val, 0, Y_MAX_NUM);
+                  x_val = CLAMP(x_val, 0, x_max_num);
+                  y_val = CLAMP(y_val, 0, y_max_num);
 
                   gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
                      p_x, p_y, x_val, y_val);
@@ -979,9 +1097,38 @@ graph_cost_year(
       fprintf(stdout, "Draw cost line %d %d %d %d\n", p_x, p_y, x_val, y_val);
 #endif
                }
-               p_x = x_val;
-               p_y = y_val;
+            }  
+            /* We need to process the Jan 1 item for mileage */
+            else if (g_date_get_year(&gdate) < year)
+            {
+               /* Get an X location that is before Jan 1 */
+               x_val = ((int)g_date_get_month(&gdate) - 12 - 1) * x_delta + X_OFFSET;
+               x_val = x_val + (x_delta * g_date_get_day(&gdate) / days_in_month);
+               process_before = 1;
             }
+            /* We need to process the Dec 31 item for mileage */
+            else if (g_date_get_year(&gdate) > year)
+            {
+               x_val = X_OFFSET + 12 * x_delta;
+               /* Process first entry of next year if we processed entries for last year */
+               if ((process_after == 0) && (p_x != -1))
+               {
+                  /* Get an X location that extents beyond Dec 31 */
+                  x_val = ((int)g_date_get_month(&gdate) + 12 - 1) * x_delta + X_OFFSET;
+                  x_val = x_val + (x_delta * g_date_get_day(&gdate) / days_in_month);
+                  y_m = y_val - p_y;
+                  x_m = x_val - p_x;
+
+                  /* y = mx + b */
+                  y_val = (y_m / x_m) * (X_OFFSET + 12 * x_delta - p_x) + p_y;
+                  x_val = X_OFFSET + 12 * x_delta;
+                  gdk_draw_line(area->window, area->style->fg_gc[GTK_WIDGET_STATE(area)],
+                     p_x, p_y, x_val, y_val);
+                  process_after = 1;
+               }
+            }
+            p_x = x_val;
+            p_y = y_val;
          }
       }
       iter = g_slist_next(iter);
@@ -989,8 +1136,11 @@ graph_cost_year(
    g_string_free(data_item, TRUE);
 }
 
-int
-graph_get_max_distance()
+void
+graph_get_max_values(
+   int* max_distance,
+   int* max_mileage,
+   int* max_cost)
 {
    GSList  *iter;
    GString *data_item;
@@ -1008,81 +1158,16 @@ graph_get_max_distance()
    char    *cost_str;
    int      dist_per_day = 0;
    int      temp;
-
-   data_item = g_string_new(NULL);
-
-   iter = sort_list;
-   while (iter != NULL)
-   {
-      g_string_erase(data_item, 0, -1);
-      g_string_append(data_item, iter->data);
-
-      date = (char *)strtok_r(data_item->str, " ", &saveptr);
-      odometer = atoi((char *)strtok_r(NULL, " ", &saveptr));
-      fuel = atof((char *)strtok_r(NULL, " ", &saveptr));
-      cost_str = (char *)strtok_r(NULL, " ", &saveptr);
-      p = (char *)strtok_r(NULL, " ", &saveptr);
-      /* IF no cost data */
-      if (p == NULL)
-      {
-         p = cost_str;
-         cost = 0.0;
-      }
-
-      g_date_set_parse(&gdate, date);
-
-      /* IF no previous entry then just save off previous values */
-      if (strcmp(p, "1") == 0)
-      {
-      }
-      else if (prev_odo == -1)
-      {
-      }
-      else
-      {
-         miles = odometer - prev_odo;
-         days = g_date_days_between(&prev_date, &gdate);
-         /* IF fillup on same day then days to 1 */
-         if (days == 0)
-            days = 1;
-         temp = (int)((double)miles / (double)days);
-         if (temp > dist_per_day)
-            dist_per_day = temp;
-      }  
-      prev_odo = odometer;
-      prev_date = gdate;
-
-      iter = g_slist_next(iter);
-   }
-   g_string_free(data_item, TRUE);
-
-#ifdef DEBUG
-      fprintf(stdout, "Max distance is %d\n", dist_per_day);
-#endif
-   return (dist_per_day);
-}
-
-int
-graph_get_max_mileage()
-{
-   GSList  *iter;
-   GString *data_item;
-   char    *date;
-   int      odometer;
-   double   fuel;
-   double   cost;
-   char    *p;
-   GDate    gdate;
-   int      prev_odo = -1;
-   char    *saveptr;
-   char    *cost_str;
-   double   max_mpg = 0.0;
-   double   mpg;
-   double   prev_fuel;
-   int      units;
    int      entry_units;
    char    *car;
+   int      units;
+   double   max_mpg = 0.0;
+   double   mpg;
+   int      cost_units;
+   double   cost_total;
+   int      local_max_cost = 0;
 
+   data_item = g_string_new(NULL);
 
    car = gtk_combo_box_get_active_text(GTK_COMBO_BOX(car_selector));
    if (car[strlen(car) - 1] == '\n')
@@ -1090,100 +1175,14 @@ graph_get_max_mileage()
       car[strlen(car) - 1] = '\0';
    }
 
-   data_item = g_string_new(NULL);
    units = returnUnits();
    entry_units = returnEntryUnits(car);
+   cost_units = returnCostUnits();
 
    /* IF entry_units are not defined then use display units */
    if (entry_units == -1)
       entry_units = units;
 
-   iter = sort_list;
-   while (iter != NULL)
-   {
-      g_string_erase(data_item, 0, -1);
-      g_string_append(data_item, iter->data);
-
-      date = (char *)strtok_r(data_item->str, " ", &saveptr);
-      odometer = atoi((char *)strtok_r(NULL, " ", &saveptr));
-      fuel = atof((char *)strtok_r(NULL, " ", &saveptr));
-      cost_str = (char *)strtok_r(NULL, " ", &saveptr);
-      p = (char *)strtok_r(NULL, " ", &saveptr);
-      /* IF no cost data */
-      if (p == NULL)
-      {
-         p = cost_str;
-         cost = 0.0;
-      }
-
-      g_date_set_parse(&gdate, date);
-
-      /* IF no previous entry then just save off previous values */
-      if (strcmp(p, "1") == 0)
-      {
-      }
-      else if (prev_odo == -1)
-      {
-      }
-      else
-      {
-         if (entry_units == CAR_MPG_US_INDEX)
-            mpg = (odometer - prev_odo) / fuel;
-         if (entry_units == CAR_MPG_IMP_INDEX)
-            mpg = (odometer - prev_odo) / fuel;
-         else if (entry_units == CAR_LP100KM_INDEX)
-            mpg = 100.0 * fuel / (odometer - prev_odo);
-         else if (entry_units == CAR_LP10KM_INDEX)
-            mpg = 10.0 * fuel / (odometer - prev_odo);
-         else if (entry_units == CAR_MPL_INDEX)
-            mpg = (odometer - prev_odo) / fuel;
-         else if (entry_units == CAR_KMPL_INDEX)
-            mpg = (odometer - prev_odo) / fuel;
-         else
-            mpg = (odometer - prev_odo) / fuel;
-         convertMPG(mpg, &mpg, entry_units, units);
-
-         if (mpg > max_mpg)
-            max_mpg = mpg;
-      }
-      prev_odo = odometer;
-      prev_fuel = fuel;
-
-      iter = g_slist_next(iter);
-   }
-   g_string_free(data_item, TRUE);
-
-   if (car)
-      free(car);
-
-#ifdef DEBUG
-      fprintf(stdout, "Max mpg is %f\n", max_mpg);
-#endif
-   return ((int)max_mpg);
-}
-
-int
-graph_get_max_cost()
-{
-   GSList  *iter;
-   GString *data_item;
-   char    *date;
-   int      odometer;
-   double   fuel;
-   double   cost;
-   char    *p;
-   GDate    gdate;
-   int      days_in_month;
-   char    *saveptr;
-   char    *cost_str;
-   int      cost_units;
-   double   cost_total;
-   int      local_max_cost = 0;
-
-
-   data_item = g_string_new(NULL);
-   cost_units = returnCostUnits();
-
    iter = sort_list;
    while (iter != NULL)
    {
@@ -1208,28 +1207,95 @@ graph_get_max_cost()
 
       g_date_set_parse(&gdate, date);
 
-      days_in_month = g_date_get_days_in_month(g_date_get_month(&gdate), g_date_get_year(&gdate));
-
-      if (cost_units == 0)
+      /* IF no previous entry then just save off previous values */
+      if (strcmp(p, "1") == 0)
+      {
+      }
+      else if (prev_odo == -1)
       {
-         if (fuel == 0)
-            cost_total = 0;
-         else
-            cost_total = (int)ceil(cost / fuel);
       }
       else
-         cost_total = cost;
+      {
+         /* Calculate distance */
+         miles = odometer - prev_odo;
+         days = g_date_days_between(&prev_date, &gdate);
+         /* IF fillup on same day then days to 1 */
+         if (days == 0)
+            days = 1;
+         temp = (int)((double)miles / (double)days);
+         if (temp > dist_per_day)
+            dist_per_day = temp;
 
-      if (cost_total > local_max_cost)
-         local_max_cost = (int)ceil(cost_total);
+         /* Calculate MPG */
+         if (entry_units == CAR_MPG_US_INDEX)
+            mpg = (odometer - prev_odo) / fuel;
+         if (entry_units == CAR_MPG_IMP_INDEX)
+            mpg = (odometer - prev_odo) / fuel;
+         else if (entry_units == CAR_LP100KM_INDEX)
+            mpg = 100.0 * fuel / (odometer - prev_odo);
+         else if (entry_units == CAR_LP10KM_INDEX)
+            mpg = 10.0 * fuel / (odometer - prev_odo);
+         else if (entry_units == CAR_MPL_INDEX)
+            mpg = (odometer - prev_odo) / fuel;
+         else if (entry_units == CAR_KMPL_INDEX)
+            mpg = (odometer - prev_odo) / fuel;
+         else
+            mpg = (odometer - prev_odo) / fuel;
+         convertMPG(mpg, &mpg, entry_units, units);
+
+         if (mpg > max_mpg)
+            max_mpg = mpg;
+
+         /* Calculate Max Cost */
+         if (cost_units == 0)
+         {
+            if (fuel == 0)
+               cost_total = 0;
+            else
+               cost_total = (int)ceil(cost / fuel);
+         }
+         else
+            cost_total = cost;
+
+         if (cost_total > local_max_cost)
+            local_max_cost = (int)ceil(cost_total);
+
+      }
+      prev_odo = odometer;
+      prev_date = gdate;
 
       iter = g_slist_next(iter);
    }
    g_string_free(data_item, TRUE);
 
-#ifdef DEBUG
-      fprintf(stdout, "Max cost is %d\n", local_max_cost);
-#endif
-   return (local_max_cost);
+   if (car)
+      free(car);
+
+   /* return values */
+   *max_distance = dist_per_day;
+   *max_mileage = max_mpg;
+   *max_cost = local_max_cost;
+}
+
+void
+graph_tab_fullscreen(int fullscreen)
+{
+   if (fullscreen == 1)
+   {
+      width_of_graph = GRAPH_WIDTH + 150;
+      height_of_graph = GRAPH_HEIGHT + 80;
+      x_delta = X_DELTA + 15;
+      y_delta = Y_DELTA + 8;
+   }
+   else
+   {
+      width_of_graph = GRAPH_WIDTH;
+      height_of_graph = GRAPH_HEIGHT;
+      x_delta = X_DELTA;
+      y_delta = Y_DELTA;
+   }
+      x_max_num = X_NUM * x_delta + X_OFFSET;
+      y_max_num = Y_NUM * y_delta + Y_OFFSET;
+      gtk_widget_set_size_request(w_area, width_of_graph, height_of_graph);
 }
 
