@@ -44,10 +44,11 @@ static void deleteCommonEntry(char *, char *, int);
  P - Mileage entry as M but no previous entry
  A - Name of car
  C - Alert - Odometer, repeat (0 if no repeat), Activity, Comments
- D - Activity - Date, Odometer, Activity, Comments
+ D - Activity - Date, Odometer, Activity, Comments, Cost
  R - Person category for Trip tab
  E - Destination category for Trip tab
  F - Trip entry
+ U - Entry Units used
 
 */
 
@@ -134,6 +135,81 @@ returnCars(char **car_array)
    return car_num;
 }
 
+GSList *
+returnCarFiles()
+{
+   GString *path_to_cars = NULL;
+   GString *path_plus_car = NULL;
+   DIR  *open_dir;
+   struct dirent  *dir;
+   char line[1024];
+   char *amount;
+   FILE *fp;
+   GSList  *lst = NULL;
+   char    *str;
+
+
+   path_to_cars = g_string_new(NULL);
+   path_plus_car = g_string_new(NULL);
+   openCarDir(path_to_cars);
+
+#ifdef DEBUG
+   fprintf(stdout, "Opening dir %s\n", path_to_cars->str);
+#endif
+   open_dir = opendir(path_to_cars->str);
+   if (open_dir)
+   {
+      while ((dir = readdir(open_dir)) != NULL)
+      {
+         if ((strcmp (dir->d_name, ".") != 0) && (strcmp(dir->d_name, "..") != 0))
+         {
+            g_string_erase(path_plus_car, 0, -1);
+            g_string_append(path_plus_car, path_to_cars->str);
+            g_string_append(path_plus_car, "/");
+            g_string_append(path_plus_car, dir->d_name);
+#ifdef DEBUG
+            fprintf(stdout, "looking at file %s\n", path_plus_car->str);
+#endif
+            fp = fopen(path_plus_car->str, "r");
+            if (fp != NULL)
+            {
+               amount = fgets(line, 1024, fp);
+               if (amount != NULL)
+               {
+#ifdef DEBUG
+                  fprintf(stdout, "first line reads:%s\n", line);
+#endif
+                  /* Is this a car file */
+                  if (strncmp(line, FIRST_LINE_STRING, strlen(FIRST_LINE_STRING)) == 0)
+                  {
+                     amount = fgets(line, 1024, fp);
+                     if (strlen(line) > 2)
+                     {
+                        /* If this is one of our files then add to car list */
+                        if (line[0] == 'A')
+                        {
+                           str = calloc(strlen(path_plus_car->str) + 1, 1);
+			   strncpy(str, path_plus_car->str, strlen(path_plus_car->str));
+#ifdef DEBUG
+                           fprintf(stdout, "adding file to list:%s\n", str);
+#endif
+                           lst = g_slist_prepend(lst, str);
+			}
+                     }
+                  }
+               }
+               fclose(fp);
+            }
+         }
+      }
+      closedir(open_dir);
+   }
+
+   g_string_free(path_to_cars, TRUE);
+   g_string_free(path_plus_car, TRUE);
+   return lst;
+}
+
 /*  This returns all mileage entries */
 GSList *
 returnMileEntries(
@@ -581,7 +657,8 @@ addActivityEntry(
    char *date_str,
    char *odo_str,
    char *act_str,
-   char *comment_str)
+   char *comment_str,
+   char *cost_str)
 {
    FILE *fp;
    char  buf[1024];
@@ -610,6 +687,12 @@ addActivityEntry(
       fwrite(buf, strlen(buf), 1, fp);
       fwrite(" ", 1, 1, fp);
       fwrite(comment_str, strlen(comment_str), 1, fp);
+      fwrite(" ", 1, 1, fp);
+
+      sprintf(buf, "%d", strlen(cost_str));
+      fwrite(buf, strlen(buf), 1, fp);
+      fwrite(" ", 1, 1, fp);
+      fwrite(cost_str, strlen(cost_str), 1, fp);
 
       fwrite("\n", 1, 1, fp);
       fclose(fp);
@@ -750,7 +833,7 @@ getNextAlert(
                }
             }
             /* ELSE a repeating event */
-	    else
+            else
             {
                /* First check to see if base odometer reading is below current */
                if (odo_int <= max_odo)
@@ -875,3 +958,101 @@ addTripEntry(
    }
 }
 
+/* This returns the entry units. -1 means none found in file */
+int
+returnEntryUnits(
+   char *car)
+{
+   char line[1024];
+   FILE *fp;
+	int  ret_int = -1;
+
+   fp = openCarFile(car);
+   if (fp != NULL)
+   {
+      fseek(fp, 0, SEEK_SET);
+      while (fgets(line, 1024, fp) != NULL)
+      {
+         if (line[strlen(line) - 1] == '\n')
+         {
+            line[strlen(line) - 1] = '\0';
+         }
+         if (line[0] == 'U')
+         {
+				ret_int = atoi(&(line[2]));
+				break;
+         }
+      }
+      fclose(fp);
+   }
+
+   return (ret_int);
+}
+
+void
+addEntryUnits(
+   char *car,
+   int   units)
+{
+   GSList  *lst = NULL;
+   char line[1024];
+   FILE *fp;
+   char *str;
+   GString *path_to_cars = NULL;
+
+   path_to_cars = g_string_new(NULL);
+
+#ifdef DEBUG
+   fprintf(stdout, "Adding Units entry %d\n", units);
+#endif
+
+   /* Put all entries into the list that don't match the specified */
+   /* units entry */
+   fp = openCarFile(car);
+   if (fp != NULL)
+   {
+      fseek(fp, 0, SEEK_SET);
+      while (fgets(line, 1024, fp) != NULL)
+      {
+         /* If there is an existing units string then skip it */
+         if (line[0] == 'U')
+         {
+         }
+         else
+         {
+            str = calloc(strlen(line) + 1, 1);
+            strcpy(str, line);
+            lst = g_slist_prepend(lst, str);
+         }
+      }
+      /* Create new entry units string */
+      sprintf(line, "U %d\n", units);
+      str = calloc(strlen(line) + 1, 1);
+      strcpy(str, line);
+      lst = g_slist_prepend(lst, str);
+
+      lst = g_slist_reverse(lst);
+      fclose(fp);
+   }
+
+   /* Now lets dump all the entries back out to the file */
+   createCarName(car, path_to_cars);
+
+   if ((fp = fopen(path_to_cars->str, "w")) != NULL)
+   {
+      GSList *next = lst;
+
+      do
+      {
+         if (next->data)
+         {
+            fwrite(next->data, strlen(next->data), 1, fp);
+            free(next->data);
+         }
+      } while ((next = g_slist_next(next)) != NULL);
+      fclose(fp);
+   }
+   g_string_free(path_to_cars, TRUE);
+   g_slist_free(lst);
+}
+
