@@ -31,5 +31,5 @@ extern int  trip_tab_get_car_inserted();
 extern void trip_tab_update_car_combo_box(int);
 extern void trip_init();
 extern void trip_set_fields(char *, char *, char *, char *, char *, char *, char *);
-extern void trip_tap_fullscreen(int fullscreen);
+extern void trip_tab_fullscreen(int fullscreen);
 #endif
